########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_actgrp.4gs/detail.4gl  1.4  Delta: 9/4/90
########################################################################

globals "globals.4gl"

#######################################################################
function lld_chkinput()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   - this function is called upon exiting the input statement.
#     the "after input" clause in the input statement can't call this
#     because a tab (control-i) needs to run this function also.
#
    define i integer        # index

# logic to check for non-null account number and null department or non-null
# department and null account number...in either case save is not allowed.

    for i = 1 to rec1_cnt
        if p_xactgd[i].acct_no is not null
        then
            if mustfill(p_xactgd[i].department, "Account Department")
            then return false end if
        end if

        if p_xactgd[i].department is not null and 
                  p_xactgd[i].department != "   "
        then 
            if mustfill(p_xactgd[i].acct_no, "Account Number")
            then return false end if
        end if

    end for

    return true
end function
# lld_chkinput()

#######################################################################
function lld_display()
#######################################################################
#   - display the first array elements onto the screen
#
    define
        n   smallint    # working number
    for n = 1 to scr1_max
        display p_xactgd[n].* to s_groupd[n].*
    end for
end function
# lld_display()

#######################################################################
function lld_p_prep(n)
#######################################################################
#   - this function creates the p_* record array (element n)
#     from the m_* record
#
    define
        n   smallint,               # array element number
        incr_with   char(2)         # account is increased with CR or DB

    let p_xactgd[n].acct_no = m_xactgd.acct_no
    let p_xactgd[n].department = m_xactgd.department
    call al_chrtn(p_xactgd[n].acct_no)
        returning p_xactgd[n].acct_desc, incr_with

end function
# lld_p_prep()

#######################################################################
function lld_m_prep(n)
#######################################################################
#   - this function creates the m_* record from the
#     p_* record array (element n) and from the elements of the
#     header record that ties the header to the detail lines.
#
    define
        n   smallint                # array element number

    # items from the header...
    let m_xactgd.grp_key = p_xactgr.grp_key

    # items from the p_ record array...
    let m_xactgd.line_no = n
    let m_xactgd.acct_no = p_xactgd[n].acct_no
    let m_xactgd.department = p_xactgd[n].department

end function
# lld_m_prep()

#######################################################################
function lld_read()
#######################################################################
#   - this function reads in all of the array elements from disk
#   - it uses a cursor, but doesn't lock any rows.
#   - it reads into the m_* record, and calls p_prep for all array elements
#
    define
        n           smallint    # working number
    whenever error continue
    declare d_cursor cursor for
        select * from stxactgd where
            stxactgd.grp_key = m_xactgr.grp_key
                order by line_no
    open d_cursor
    let n = 0
    foreach d_cursor into m_xactgd.*
        let n = n + 1
        if n > rec1_max
        then
            call fg_err(1) # Too many rows in stgdistd to read.
            let rec1_cnt = 0
            call set_count(rec1_cnt)
            close d_cursor
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(n)                  # move data from m_* to p_[].*
    end foreach
    let rec1_cnt = n
    call set_count(rec1_cnt)
    close d_cursor
    return
end function
# lld_read()

#######################################################################
function lld_delete()
#######################################################################
#   - this function is for deleting all rows in the array
#
    delete from stxactgd where
            stxactgd.grp_key = m_xactgr.grp_key
end function
# lld_delete()

#######################################################################
function lld_add()
#######################################################################
#   - this function is for writing all rows in the array
#
    define
        n   smallint    # working number
    whenever error continue
    for n = 1 to rec1_cnt       # for all active array elements
        call lld_m_prep(n)
        insert into stxactgd values(m_xactgd.*)
    end for
    whenever error call error_handler
    return
end function
# lld_add()
