##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_actrng.4gs/header.4gl  1.3  Last Delta: 8/31/90
##############################################################################

globals "globals.4gl"

#######################################################################
function llh_input()
# returning -1 if tab pressed, 0 otherwise
#######################################################################
#
    input p_xcntrc.* without defaults from s_cntrc.*
        on key (control-e)
            call fg_err(0)
        on key (control-w)
            call fg_help(1)
        on key (control-f)
            # add/update user defined fields...
            call al_addlz("stxcntrc","1","","")
    end input
    return 0
end function
# llh_input()

#######################################################################
function llh_chkinput()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   this function is called upon exiting the input statement.
#   the "after input" clause in the input statement can't call
#   this because a tab (control-i) needs to run this function also.
#
#   if you have any must-fill type fields on the screen, you can use
#   the function "mustfill()" that comes with infourgen.  it's syntax is:
#       mustfill(variable_name,"description of that variable")
#           returning true/false based upon no_data/data in that field.
#   it's use in this context is:
#       if mustfill(p_xchrt.acct_no,"Account Number") then return false end if
#

    if mustfill(p_xcntrc.curr_asset,"Current Asset") then return false end if
    if mustfill(p_xcntrc.fixed_asset,"Fixed Asset") then return false end if
    if mustfill(p_xcntrc.curr_liab,"Current Liability") then return false end if
    if mustfill(p_xcntrc.long_term_liab,"Long Term Liability") then return false end if
    if mustfill(p_xcntrc.capital,"Capital") then return false end if
    if mustfill(p_xcntrc.income,"Income") then return false end if
    if mustfill(p_xcntrc.cost_goods,"Cost of Goods") then return false end if
    if mustfill(p_xcntrc.expense,"Expense") then return false end if
    if
      p_xcntrc.curr_asset < p_xcntrc.fixed_asset and
      p_xcntrc.fixed_asset < p_xcntrc.curr_liab and
      p_xcntrc.curr_liab < p_xcntrc.long_term_liab and
      p_xcntrc.long_term_liab < p_xcntrc.capital and
      p_xcntrc.capital < p_xcntrc.income and
      p_xcntrc.income < p_xcntrc.cost_goods and
      p_xcntrc.cost_goods < p_xcntrc.expense
    then
        # accounts are properly ordered
    else
        call fg_err(1) # Account ranges must be ascending
        return false
    end if
    return true
end function
# llh_chkinput()

#######################################################################
function llh_display()
#######################################################################
#   - display all p_* variables onto the screen
#
    display p_xcntrc.* to s_cntrc.*
end function
# llh_display()

#######################################################################
function llh_p_prep()
#######################################################################
#   - this function creates the p_* record from the m_* record
#
    let p_xcntrc.curr_asset = m_xcntrc.curr_asset
    let p_xcntrc.fixed_asset = m_xcntrc.fixed_asset
    let p_xcntrc.curr_liab = m_xcntrc.curr_liab
    let p_xcntrc.long_term_liab = m_xcntrc.long_term_liab
    let p_xcntrc.capital = m_xcntrc.capital
    let p_xcntrc.income = m_xcntrc.income
    let p_xcntrc.cost_goods = m_xcntrc.cost_goods
    let p_xcntrc.expense = m_xcntrc.expense
end function
# llh_p_prep()

#######################################################################
function llh_m_prep()
#######################################################################
#   - this function creates the m_* record from the p_* record
#
    let m_xcntrc.curr_asset = p_xcntrc.curr_asset
    let m_xcntrc.fixed_asset = p_xcntrc.fixed_asset
    let m_xcntrc.curr_liab = p_xcntrc.curr_liab
    let m_xcntrc.long_term_liab = p_xcntrc.long_term_liab
    let m_xcntrc.capital = p_xcntrc.capital
    let m_xcntrc.income = p_xcntrc.income
    let m_xcntrc.cost_goods = p_xcntrc.cost_goods
    let m_xcntrc.expense = p_xcntrc.expense
end function
# llh_m_prep()

#######################################################################
function llh_read(ring_rowid)
#######################################################################
#   - this function reads in all of the p_* variables from disk.
#   - it does not use a cursor, so it doesn't lock the row.
#
    define
        ring_rowid  integer     # rowid of record to read
    whenever error continue
    select * into m_xcntrc.* from stxcntrc where rowid = ring_rowid
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        return
    end if
    call llh_p_prep()
    return
end function
# llh_read()

#######################################################################
function llh_delete(ring_rowid)
#######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxcntrc
        where rowid = ring_rowid
end function
# llh_delete()

#######################################################################
function llh_add()
#######################################################################
#   - insert a new reference record, and all n-lines associated with it.
#
    whenever error continue
    insert into stxcntrc values(m_xcntrc.*)
    whenever error call error_handler
end function
# llh_add()

#######################################################################
function llh_update(ring_rowid)
#######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxcntrc set stxcntrc.* = m_xcntrc.*
        where rowid = ring_rowid
end function
# llh_update()
