##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_actrng.4gs/midlevel.4gl  1.1  Last Delta: 10/25/88
##############################################################################

globals "globals.4gl"

#######################################################################
# "ok" functions:
#  - the following "ok" functions can be placed here:
#    ok_add(), ok_update(), ok_delete(), ok_find(), ok_browse(), ok_next(),
#    ok_prev(), ok_tab(), ok_options(), ok_exit(), and ok_bang().
#  - their purpose is to allow/disallow that function from the ring menu.
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#  - following is an example of a smart delete function:
#
#    ##################################################################
#    function ok_delete()
#    # returning true or false based upon ok to delete
#    ##################################################################
#    #
#        if posted = "Y"
#        then
#            error " Cannot delete a posted record "
#            return false
#        end if
#        return true
#    end function
#    # ok_delete()
#
#######################################################################

#######################################################################
# "ok" functions:
#  - there is an "ok" function for each of the ring_menu options.
#  - their purpose is to allow/disallow that function
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#######################################################################

#######################################################################
function ok_add()
# returning true/false based upon ok/not_ok to add
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_add()

#######################################################################
function ok_find()
# returning true/false based upon ok/not_ok to find
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_find()

#######################################################################
function ok_delete()
# returning true/false based upon ok/not_ok to delete
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_delete()

#######################################################################
function mlh_init()
# returning the where clause string
#######################################################################
#  - this functions only purpose is to return a string that will be used
#    for the where clause in the initial document grouping that takes
#    place when the program is loaded.
#  - examples:
#   return "1=1" (select all rows) #
#   return "1=2" (select no rows)
#   return "posted = \"N\" " (select only unposted documents) #
#   return "rowid = 1" (select only the first row - for control files) #
#
    return "1=1"
end function
# mlh_init()

#######################################################################
function mlh_validate()
#######################################################################
#   mid level data validation.  this function is called before any
#   disk writes.  if informix tries to write invalid data, part of the row
#   will be written, and part of it won't.  sql bomb's out.  the data
#   should have been checked in ll_input(), but us programmers can't
#   be perfect all of the time.  instead of data being compromised, the
#   end user doesn't get to write that screen.  it ticks them off, but
#   at least they don't have partial records on disk.
#
    whenever error continue
    call llh_m_prep()
    validate m_xcntrc.* like stxcntrc.*
    whenever error call error_handler
end function
# mlh_validate()

#######################################################################
function mlh_clear()
#######################################################################
#
    initialize p_xcntrc.* to null
    initialize m_xcntrc.* to null
end function
# mlh_clear

#######################################################################
function mlh_construct()
# returning criteria
#######################################################################
#
    define
        criteria    char(256)   # selection criteria holding bin
    # construct the search criteria from the screen
    construct criteria
        on
            stxcntrc.curr_asset,
            stxcntrc.fixed_asset,
            stxcntrc.curr_liab,
            stxcntrc.long_term_liab,
            stxcntrc.capital,
            stxcntrc.income,
            stxcntrc.cost_goods,
            stxcntrc.expense
        from
            s_cntrc.curr_asset,
            s_cntrc.fixed_asset,
            s_cntrc.curr_liab,
            s_cntrc.long_term_liab,
            s_cntrc.capital,
            s_cntrc.income,
            s_cntrc.cost_goods,
            s_cntrc.expense
    return criteria
end function
# mlh_construct

#######################################################################
function mlh_define_cur(criteria)
#######################################################################
#
    define
        criteria    char(256),  # selection criteria holding bin
        find_specs  char(512),  # select sql specification
        curs_count  integer     # number of elements in the cursor
    # get the count into curs_count
    let find_specs =
        "select count(*) from stxcntrc where ",
        criteria clipped
    prepare get_count from find_specs
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into curs_count
    close count_cursor
    # define the cursor
    let find_specs =
        "select rowid",
        " from stxcntrc where ",criteria clipped,
        " "
    prepare get_stmt from find_specs
    declare ref_cursor scroll cursor for get_stmt
    open ref_cursor
    let sqlca.sqlerrd[3] = curs_count
    return
end function
#mlh_define_cur()


#######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
#######################################################################
#
    define
        curno       integer,    # absolute cursor number
        curs_rowid integer
    fetch absolute curno ref_cursor into
            curs_rowid
    let sqlca.sqlerrd[6] = curs_rowid
    return
end function
# mlh_fetch_cur()

#######################################################################
function mlh_close_cur()
#######################################################################
#
    close ref_cursor
end function
# mlh_close_cur()

#######################################################################
function mlh_lock(ring_rowid)
#######################################################################
#   - this function opens a cursor for update selecting just the
#     record defined by rowid.
#   - because it fetches the record with an update cursor, the record
#     is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        ring_rowid  integer,    # rowid of record to read
        getid       integer     # something to fetch into
    whenever error continue
    declare lock_cursor cursor for
        select rowid into getid from stxcntrc where rowid = ring_rowid
        for update
    open lock_cursor
    fetch lock_cursor
    whenever error call error_handler
    return
end function
# mlh_lock()

#######################################################################
function mlh_unlock()
#######################################################################
#
    close lock_cursor
end function
# mlh_unlock()

