##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_cashe.4gs/midlevel.4gl  1.1  Last Delta: 11/24/89
##############################################################################

globals "globals.4gl"

#######################################################################
# "ok" functions:
#  - the following "ok" functions can be placed here:
#    ok_add(), ok_update(), ok_delete(), ok_find(), ok_browse(), ok_next(),
#    ok_prev(), ok_tab(), ok_options(), ok_exit(), and ok_bang().
#  - their purpose is to allow/disallow that function from the ring menu.
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#  - following is an example of a smart delete function:
#
#    ##################################################################
#    function ok_delete()
#    # returning true or false based upon ok to delete
#    ##################################################################
#    #
#        if posted = "Y"
#        then
#            error " Cannot delete a posted record "
#            return false
#        end if
#        return true
#    end function
#    # ok_delete()
#
#######################################################################

##################################################################
function ok_add()
# returning true or false based upon ok to add
##################################################################
# assign rowid to null
#
    let p_rowid = null
    return true
end function
# ok_add()

##################################################################
function ok_update()
# returning true or false based upon ok to update
##################################################################
# assign rowid to null
#
    call fg_err(5) # cannot update definition, use delete to remove definition
    return false
end function
# ok_update()

##################################################################
function ok_delete()
# returning true or false based upon ok to delete
##################################################################
# assign rowid to null
#
    define
        prompt_response char(1)

    open window delete_rec at 14,16 with 9 rows, 51 columns
        attribute (border, blue, prompt line last)
    display "===================================================" at 2,1
    display " Deletion of this record will also delete all      " at 3,1
    display " check register transaction records associated     " at 4,1
    display " with this account number and department.          " at 5,1
    display "===================================================" at 8,1

    let prompt_response = null
    while prompt_response matches "[^yYnN]" or
          prompt_response is null
        prompt " Continue? " for prompt_response
    end while
    if upshift(prompt_response) = "N"
    then
        close window delete_rec
        return false
    end if

    close window delete_rec
    return true

end function
# ok_delete()

#######################################################################
function mlh_init()
# returning the where clause string
#######################################################################
#  - this functions only purpose is to return a string that will be used
#    for the where clause in the initial document grouping that takes
#    place when the program is loaded.
#  - examples:
#   return "1=1" (select all rows)
#   return "1=2" (select no rows)
#   return "posted = 0" (select only unposted documents)
#   return "rowid = 1" (select only the first row - for control files)
#
    return "1=2"
end function
# mlh_init()

#######################################################################
function mlh_validate()
#######################################################################
#   mid level data validation.  this function is called before any
#   disk writes.  if informix tries to write invalid data, part of the row
#   will be written, and part of it won't.  sql bomb's out.  the data
#   should have been checked in ll_input(), but us programmers can't
#   be perfect all of the time.  instead of data being compromised, the
#   end user doesn't get to write that screen.  it ticks them off, but
#   at least they don't have partial records on disk.
#
    whenever error continue
    call llh_m_prep()
    validate m_xckrgr.* like stxckrgr.*
    whenever error call error_handler
end function
# mlh_validate()

#######################################################################
function mlh_clear()
#######################################################################
#
    initialize p_xckrgr.* to null
    initialize m_xckrgr.* to null
end function
# mlh_clear

#######################################################################
function mlh_construct()
# returning criteria
#######################################################################
#
    define
        criteria    char(256)   # selection criteria holding bin
    # construct the search criteria from the screen
    construct criteria
        on
            stxckrgr.acct_no,
            stxckrgr.department
        from
            s_xckrgr.acct_no,
            s_xckrgr.department
    return criteria
end function
# mlh_construct

#######################################################################
function mlh_define_cur(criteria)
#######################################################################
#
    define
        criteria    char(256),  # selection criteria holding bin
        find_specs  char(512),  # select sql specification
        curs_count  integer     # number of elements in the cursor

    if criteria = "1=2" then let sqlca.sqlerrd[3] = 0 return end if

    # get the count into curs_count
    let find_specs =
        "select count(*) from stxckrgr where ",
        criteria clipped
    prepare get_count from find_specs
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into curs_count
    close count_cursor
    # define the cursor
    let find_specs =
        "select rowid, acct_no, department",
        " from stxckrgr where ",criteria clipped,
        " order by acct_no, department"
    prepare get_stmt from find_specs
    declare ref_cursor scroll cursor for get_stmt
    open ref_cursor
    let sqlca.sqlerrd[3] = curs_count
    return
end function
#mlh_define_cur()


#######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
#######################################################################
#
    define
        curno       integer,    # absolute cursor number
        curs_rowid integer, p1 char(1), p2 char(1)

    if curno = 0 then let sqlca.sqlerrd[6] = -1  return end if
    fetch absolute curno ref_cursor into curs_rowid, p1, p2
    let sqlca.sqlerrd[6] = curs_rowid
    let p_rowid = curs_rowid
    return
end function
# mlh_fetch_cur()

#######################################################################
function mlh_close_cur()
#######################################################################
#
    close ref_cursor
end function
# mlh_close_cur()

#######################################################################
function mlh_lock(ring_rowid)
#######################################################################
#   - this function opens a cursor for update selecting just the
#     record defined by rowid.
#   - because it fetches the record with an update cursor, the record
#     is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        ring_rowid  integer,    # rowid of record to read
        getid       integer     # something to fetch into
    whenever error continue
    declare lock_cursor cursor for
        select rowid into getid from stxckrgr where rowid = ring_rowid
        for update
    open lock_cursor
    fetch lock_cursor
    whenever error call error_handler
    return
end function
# mlh_lock()

#######################################################################
function mlh_unlock()
#######################################################################
#
    close lock_cursor
end function
# mlh_unlock()

