########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_chglog.4gs/main.4gl  1.4  Delta: 8/31/90
########################################################################

globals "globals.4gl"

#######################################################################
main
#######################################################################
#
    options input wrap
    options prompt line first + 1
    options form line first + 3
    let progid = "all.i_chglog"
    clear screen
    whenever error call error_handler
    open window win1 at 2,3 with 22 rows, 76 columns
        attribute (border, red)
    call logo()
    clear window win1
    open window win2 at 2,3 with 22 rows, 76 columns
        attribute (blue)
    open form screen1 from "screen1"
    display form screen1
    call get_logpath()
    call update_logpath()
    exit program (0)

end main

#######################################################################
function get_logpath()
#######################################################################
#
# this function is to get the logpath assigned to the specified database
# if the system is using on-line or turbo, the log tracking must be
# done by the system administrator.
# the functin will get the logpath from stxcompr for 
# the specified database first, if the system is using 
# informix 4.0 or later, the function will get the logfile 
# from the table systables
#
    define
        n smallint           # working number

    # default to the standard database
    let compid  = null
    let dbname  = null
    let logpath = null

    # parse the command line for "-c $company" where company is
    # the company id that is used as a key into stxcompr to find
    # the database name, and the logfile path.
    for n = 1 to num_args() - 1
        if arg_val(n) = "-c"
        then
            let compid = arg_val(n+1)
        end if
        if arg_val(n) = "-f"
        then
            let fg_variable = arg_val(n+1)
        end if
    end for

    # try to find the database name in stxcompr
    if compid is not null
    then
        let no_company = false
        whenever error continue
        select db_name, logfile into  dbname, logpath
            from stxcompr where comp_id = compid
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let no_company = true
            # try opening the database to see if it exists
            whenever error continue
            database compid
            whenever error call error_handler
            if sqlca.sqlcode = 0
            then 
                let dbname = compid
            else 
                call fg_err(1) # Specified company not found
                exit program(1)
            end if
        end if
    end if

    close database
    database dbname

    # check the table syscolumns to see if the column dirpath exists in
    # systables.  if it does not, then either turbo or on-line 
    # are in use and it is up to the system administrator to 
    # turn logging on and off.  

    select rowid from syscolumns where tabid = 1 and colname = "dirpath"

    if sqlca.sqlcode = notfound 
    then 
        call fg_err(2) # turbo or on-line in use, must user monitor to set logging
        exit program(1)
    end if

    # check systables for the column viewtext in table sysviews.  if it
    # exists, then a 4.0 or later standard engine is in use and logging
    # is always turned on.  if 4.0 or later standard engine is used then
    # use the dirpath in systables as the logpath

    let four_or_later = false

    select rowid from syscolumns 
      where tabid = 6 and colname = "viewtext"

    if sqlca.sqlcode = 0 
    then
        let four_or_later = true
        select dirpath into p_logpath from systables 
           where tabname = "syslog"
        if sqlca.sqlcode = 0   # if dirpath is found in systables
        then
            let logpath = p_logpath
        end if 
    end if

end function
# get_logpath

#######################################################################
function update_logpath()
#######################################################################
#
# this function allows the user to update the path of the logfile,
# if informix version is 4.0 or late this function will start 
# the database with the new log file
# update the logfile column in stxcompr in the standard database
#
    define 
      sql_stmt char(128),
      quote char(1)

    let quote = "\""

	display "Update:  [ESC] to Store  [DEL] to Cancel" at 1,2
	display "Enter changes into form" at 2,2
    call menu_line()

    display logpath to s_screen.oldlogfile

    input p_logpath without defaults from s_screen.logfile
      after field logfile
        if p_logpath is null
        then
          let p_logpath = fg_variable clipped, "/accounting/data/inx_log"
          display p_logpath to s_screen.logfile
        end if
    end input

    if int_flag != 0 then return end if

    if logpath = p_logpath then return end if

    if four_or_later
    then
        close database
        let sql_stmt = 
          "start database ", dbname, 
            " with log in ", quote, p_logpath, quote
        prepare start_db from sql_stmt
        execute start_db
    end if

    close database
    database standard
    whenever error continue
    if no_company
    then
        insert into stxcompr 
          (db_name, logfile, comp_id) values (dbname, p_logpath, compid)
    else
        update stxcompr set logfile = p_logpath where comp_id = compid
    end if
    whenever error call error_handler

end function
# update_logpath()
