########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_contrl.4gs/detail.4gl  1.6  Delta: 9/4/90
########################################################################

globals "globals.4gl"

#######################################################################
function lld_input()
# returning -1 if tab pressed, 0 otherwise
#######################################################################
#   - scrolling input array
#   - must call set_count before input array
#
    define
        n     smallint,
        pcurr smallint,
        scurr smallint

    call set_count(rec1_cnt)    # this must be called before input array
    input array p_xinfor without defaults from s_xinfor.*
        on key (control-e)
            call fg_err(0)
        on key (control-w)
            call fg_help(2)
        on key (control-f)
            # add/update user defined fields...
            call al_addlz("stxcntrc","1","","")
        after delete
            let n = arr_count() + 1
            initialize p_xinfor[n].* to null
        before field src_key
            let pcurr = arr_curr()
            if p_xinfor[pcurr].src_key is not null and
               p_xinfor[pcurr].src_key = "000"
            then
                    next field src_desc 
            end if

        after field src_key
            let pcurr = arr_curr()
            let scurr = scr_line()
            if chkdup()
            then
                let p_xinfor[pcurr].src_key = ""
                display p_xinfor[pcurr].src_key
                     to s_xinfor[scurr].src_key
                next field src_key
            end if
        on key (control-i)
            return -1
    end input
    return 0
end function
# lld_input()

#######################################################################
function lld_chkinput()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   - this function is called upon exiting the input statement.
#     the "after input" clause in the input statement can't call this
#     because a tab (control-i) needs to run this function also.
#
    if chkdup()
    then
        return false
    end if
    return true
end function
# lld_chkinput()

#######################################################################
function lld_display()
#######################################################################
#   - display the first array elements onto the screen
#
    define
        n   smallint    # working number
    for n = 1 to scr1_max
        display p_xinfor[n].* to s_xinfor[n].*
    end for
end function
# lld_display()

#######################################################################
function lld_p_prep(n)
#######################################################################
#   - this function creates the p_* record array (element n)
#     from the m_* record
#
    define
        n   smallint                # array element number

    let p_xinfor[n].src_key = m_xinfor.src_key
    let p_xinfor[n].src_desc = m_xinfor.src_desc

end function
# lld_p_prep()

#######################################################################
function lld_m_prep(n)
#######################################################################
#   - this function creates the m_* record from the
#     p_* record array (element n) and from the elements of the
#     header record that ties the header to the detail lines.
#
    define
        n   smallint                # array element number

    # items from the header...
    let m_xinfor.src_type = "D"

    # items from the p_ record array...
    let m_xinfor.src_key = p_xinfor[n].src_key
    let m_xinfor.src_desc = p_xinfor[n].src_desc

end function
# lld_m_prep()

#######################################################################
function lld_read()
#######################################################################
#   - this function reads in all of the array elements from disk
#   - it uses a cursor, but doesn't lock any rows.
#   - it reads into the m_* record, and calls p_prep for all array elements
#
    define
        n           smallint    # working number
    whenever error continue
    declare d_cursor cursor for
        select * from stxinfor where
            stxinfor.src_type = "D"
        order by stxinfor.src_key
    open d_cursor
    let n = 0
    foreach d_cursor into m_xinfor.*
        let n = n + 1
        if n > rec1_max
        then
            call fg_err(1) # Too many department rows in stxinfor to read
            let rec1_cnt = 0
            call set_count(rec1_cnt)
            close d_cursor
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(n)                  # move data from m_* to p_[].*
    end foreach
    let rec1_cnt = n
    call set_count(rec1_cnt)
    close d_cursor
    return
end function
# lld_read()

#######################################################################
function lld_delete()
#######################################################################
#   - this function is for deleting all rows in the array
#
    delete from stxinfor where
            stxinfor.src_type = "D"
end function
# lld_delete()

#######################################################################
function lld_add()
#######################################################################
#   - this function is for writing all rows in the array
#
    define
        n   smallint    # working number
    whenever error continue
    for n = 1 to rec1_cnt       # for all active array elements
        call lld_m_prep(n)
        insert into stxinfor values(m_xinfor.*)
    end for
    whenever error call error_handler
    return
end function
# lld_add()

#######################################################################
function chkdup()
#   returns true/false based upon dup/no dup
#######################################################################
#   - This checks for duplicate department src_key's in the info file.
#

    define
        curr_line smallint,     # current line
        n         smallint      # working number

    # setup the number of elements into rec1_cnt
    call mld_arr_count()
    let curr_line = arr_curr()

    for n = 1 to rec1_max
        if p_xinfor[n].src_key = p_xinfor[curr_line].src_key and
           n != curr_line
        then
            call fg_err(2) # Department Code already been defined
            return true
        end if
    end for
    return false
end function
# chkdup()
