##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_contrl.4gs/midlevel.4gl  1.2  Last Delta: 11/28/89
##############################################################################

globals "globals.4gl"

#######################################################################
# "ok" functions:
#  - there is an "ok" function for each of the ring_menu options.
#  - their purpose is to allow/disallow that function
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#######################################################################

#######################################################################
function ok_add()
# returning true/false based upon ok/not_ok to add
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_add()

#######################################################################
function ok_find()
# returning true/false based upon ok/not_ok to find
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_find()

#######################################################################
function ok_delete()
# returning true/false based upon ok/not_ok to delete
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_delete()

#######################################################################
function mlh_validate()
#######################################################################
#   mid level data validation.  this function is called before any
#   disk writes.  if informix tries to write invalid data, part of the row
#   will be written, and part of it won't.  sql bomb's out.  the data
#   should have been checked in ll_input(), but us programmers can't
#   be perfect all of the time.  instead of data being compromised, the
#   end user doesn't get to write that screen.  it ticks them off, but
#   at least they don't have partial records on disk.
#
    whenever error continue
    call llh_m_prep()
    validate m_xcntrc.* like stxcntrc.*
    whenever error call error_handler
end function
# mlh_validate()

#######################################################################
function mld_validate()
#######################################################################
#   like above, only validation must occur for each line
#
    define
        n   smallint    # working number
    whenever error continue
    for n = 1 to rec1_cnt
        call lld_m_prep(n)
        validate m_xinfor.* like stxinfor.*
    end for
    whenever error call error_handler
end function
# mld_validate

#######################################################################
function mlh_clear()
#######################################################################
#
    initialize p_xcntrc.* to null
    initialize m_xcntrc.* to null
end function
# mlh_clear

#######################################################################
function mld_clear()
#######################################################################
#
    define
        n       smallint            # working number
    initialize m_xinfor.* to null    # clear the m_record
    # clear all p_ records in the array
    for n = 1 to rec1_max
        initialize p_xinfor[n].* to null   # clear the p_record
    end for
    let rec1_cnt = 0                # reset the number of active elements
end function
# mld_mclear()

#######################################################################
function mlh_construct()
# returning criteria
#######################################################################
#
    define
        criteria    char(256)   # selection criteria holding bin
    # construct the search criteria from the screen
    construct criteria
        on
            stxcntrc.co_name,
            stxcntrc.addr1,
            stxcntrc.addr2,
            stxcntrc.city,
            stxcntrc.state,
            stxcntrc.zip,
            stxcntrc.county,
            stxcntrc.country
        from
            s_xcntrc.co_name,
            s_xcntrc.addr1,
            s_xcntrc.addr2,
            s_xcntrc.city,
            s_xcntrc.state,
            s_xcntrc.zip,
            s_xcntrc.county,
            s_xcntrc.country
    return criteria
end function
# mlh_construct

#######################################################################
function mlh_define_cur(criteria)
#######################################################################
#
    define
        criteria    char(256),  # selection criteria holding bin
        find_specs  char(512),  # select sql specification
        curs_count  integer     # number of elements in the cursor
    # get the count into curs_count
    let find_specs =
        "select count(*) from stxcntrc where ",
        criteria clipped
    prepare get_count from find_specs
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into curs_count
    close count_cursor
    # define the cursor
    let find_specs =
        "select rowid",
        " from stxcntrc where ",criteria clipped,
        " "
    prepare get_stmt from find_specs
    declare ref_cursor scroll cursor for get_stmt
    open ref_cursor
    let sqlca.sqlerrd[3] = curs_count
    return
end function
#mlh_define_cur()


#######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
#######################################################################
#
    define
        curno       integer,    # absolute cursor number
        curs_rowid integer
    fetch absolute curno ref_cursor into
            curs_rowid
    let sqlca.sqlerrd[6] = curs_rowid
    return
end function
# mlh_fetch_cur()

#######################################################################
function mlh_close_cur()
#######################################################################
#
    close ref_cursor
end function
# mlh_close_cur()

#######################################################################
function mlh_lock(ring_rowid)
#######################################################################
#   - this function opens a cursor for update selecting just the
#     record defined by rowid.
#   - because it fetches the record with an update cursor, the record
#     is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        ring_rowid  integer,    # rowid of record to read
        getid       integer     # something to fetch into
    whenever error continue
    declare lock_cursor cursor for
        select rowid into getid from stxcntrc where rowid = ring_rowid
        for update
    open lock_cursor
    fetch lock_cursor
    whenever error call error_handler
    return
end function
# mlh_lock()

#######################################################################
function mlh_unlock()
#######################################################################
#
    close lock_cursor
end function
# mlh_unlock()

#######################################################################
function mld_scroll()
#######################################################################
#   - this function calls display array for the window.
#
    if rec1_cnt < 1
    then
        return
    end if
    call set_count(rec1_cnt)
    display array p_xinfor to s_xinfor.*
        on key (control-i)
            exit display
    end display
end function
# mld_scroll

##########################################################################
function mld_arr_count()
##########################################################################
#   - this function sets the rec1_cnt variable instead of informix's
#     arr_count() function that can't be trusted.
#
    define
        n       smallint        # working number
    for n = rec1_max to 1 step -1
        if      p_xinfor[n].src_key  is not null
            or  p_xinfor[n].src_desc  is not null
        then
          if n <= arr_count()
          then
            let rec1_cnt = n
            return
          else
            let rec1_cnt = arr_count()
            initialize p_xinfor[rec1_cnt+1].* to null   # clear the p_record
            return
          end if
        end if
    end for
    let rec1_cnt = 0
    return
end function
# mld_arr_count()

